define("wagby/portlet/CalendarPortlet", [
  "dojo/_base/declare",
  "dojo/date",
  "dojo/date/locale",
  "dojo/date/stamp",
  "dojo/dom-construct",
  "dojo/request",
  "dojo/store/Memory",
  "dojo/store/Observable", 
  "dojox/calendar/Calendar", 
  "wagby/portlet/BasePortlet"
], function(declare, date, locale, stamp, domConstruct, request, Memory, Observable, Calendar, BasePortlet) {
  return declare("wagby.portlet.CalendarPortlet", [BasePortlet], {
    calendarUrl: "",

    postCreate: function() {
      this.inherited(arguments);
    },

    getCalendarOption: function() {
      // スクリプト欄にカレンダの設定を返す関数があれば実行する
      if (!jfcCalendarPortletOption) {
        return {};
      }
      return jfcCalendarPortletOption();
    },

    showContents: function() {
      this.inherited(arguments);
      this.createCalendar();

      this.calendarUrl = this.getURL() + "calendar/";

      // 初期表示のために今月分のカレンダデータを取得する
      this.getCalendarData(new Date());
    },

    createCalendar: function() {
      calendarStore = new Memory();
      // カレンダのオプションをカレンダポートレットのスクリプト欄から取得する
      var calendarOption = this.getCalendarOption();
      // カレンダの表示サイズをポートレットのサイズに設定する
      var width = this.portletWidth - 25;
      var height = this.portletHeight - 60;
      // カレンダ生成
      try {
        this.calendar = new Calendar({
          dateInterval: calendarOption.dateInterval || "week",
          columnViewProps: {
            minHours: this.getHourValue(calendarOption.minHours, 8),
            maxHours: this.getHourValue(calendarOption.maxHours, 24),
            columnHeaderDatePattern: 'MM/dd'
          },
          editable: false,
          style: "position: relative; width: " + width + "px; height: " + height + "px;"
        }, this.contentsArea);
      } catch(e) {
        console.error(e);
      }
//      calendar.fourDaysButton.set("disabled", true);

      var that = this;
      // 週表示のカラムでは初期表示時に時間を現在時刻に合わせる
      this.calendar.on("RenderersLayoutDone", function(view) {
        that.calendar.columnView.set('startTimeOfDay', {hours: new Date().getHours(), duration: 10});
      });
      // カレンダの次ボタン押下時の処理
      this.calendar.nextButton.on("Click", function() {
        that.getNextOrPreviousCalendarData();
      });
      // カレンダの前ボタン押下時の処理
      this.calendar.previousButton.on("Click", function() {
        that.getNextOrPreviousCalendarData();
      });
      // カレンダの表示月変更時に開始日でカレンダのデータを取得する
      this.calendar.on("TimeIntervalChange", function() {
        that.getNextOrPreviousCalendarData();
      });
      // モデル毎のカレンダの色を設定する
      this.setCalendarCssClassFunc(this.calendar, calendarOption);
    },

    getHourValue: function(value, defaultValue) {
      if (typeof value === 'undefined' || value === null || value !== value) {
        return defaultValue;
      }
      return value;
    },

    getNextOrPreviousCalendarData: function() {
      // カレンダの前・次ボタン押下後の日付を元にカレンダのデータを取得する
      var url = this.calendarUrl + this.calendar.date.getFullYear() + "-" +
        (this.calendar.date.getMonth() + 1) + "-01";
      this.getCalendarItems(url);
    },

    setCalendarCssClassFunc: function(calendar, calendarOption) {
      calendar.set("cssClassFunc", function(item) {
        // 休日モデルのデータは「Calendar1」がセットされている
        if (item.calendar) {
          return item.calendar;
        }
        // その他のモデルの場合、ポートレット設定の calendar 属性で設定された値を返す
        // 設定無しの場合は、デフォルトの色「Calendar2」(青)を返す
        return calendarOption.calendar || "Calendar2";
      });
    },

    getCalendarUrl: function(dateObj) {
      var year = dateObj.getFullYear();
      var month = dateObj.getMonth() + 1;
      return this.calendarUrl + year + "-" + month + "-01";
    },

    getCalendarData: function(dateObj) {
      this.getCalendarItems(this.getCalendarUrl(dateObj));
    },

    // カレンダの表示内容を取得する
    getCalendarItems: function(url) {
      var that = this;
      request(url, {
        headers: {
          'X-Wagby-RESTAPIVersion': "v2",
        },
        handleAs: "json",
        withCredentials: true,
      }).then(function(data) {
        var calendarItems = [];
        var items = data.listp.item || data.listp.item_;
        for (var i=0; i<items.length; i++) {
          var item = items[i];
          var calitems = item.calendaritem || item.calendaritem_;
          if (calitems.length === 0) {
            continue;
          }
          for (var j=0; j<calitems.length;j++) {
            var calendaritem = calitems[j];
            var idate = item.date || item.date_;
            var itemDate = idate.split("-");

            var startTime = that.getCalendaritemStartTime(itemDate, calendaritem);
            var allDay = false;
            var starthour = calendaritem.starthour || calendaritem.starthour_;
            if (!starthour) {
              allDay = true;
            }
            var endTime = that.getCalendaritemEndTime(itemDate, calendaritem, allDay);
            calendarItems.push({
              "id": (calendaritem.modelid || calendaritem.modelid_) + ":" + calendaritem.calendarid,
              "summary": calendaritem.title || calendaritem.title_,
              "startTime": startTime,
              "endTime": endTime,
              "allDay": allDay,
              "calendar": calendaritem.modelid === "jholiday" ? "Calendar1" : null,
            });
          }
        }
        calendarStore.setData(calendarItems);
        that.calendar.set("store", new Observable(calendarStore));
      }, function(err) {
console.log("schedule err: ", err);
      });
    },

    getCalendaritemStartTime: function(itemDate, calendaritem) {
      var startTime = new Date(itemDate[0], (parseInt(itemDate[1], 10) - 1), itemDate[2]);
      var starthour = calendaritem.starthour;
      if (Number.isInteger(starthour)) {
        startTime.setHours(starthour);
      } else {
        startTime.setHours(0);
      }
      var startminute = calendaritem.startminute;
      if (Number.isInteger(startminute)) {
        startTime.setMinutes(startminute);
      } else {
        startTime.setMinutes(0);
      }
      var startTimeString = stamp.toISOString(startTime, {zulu: false});
      startTimeString = startTimeString.substring(0, 19);
      return startTimeString;
    },

    getCalendaritemEndTime: function(itemDate, calendaritem, allDay) {
      var endTime = new Date(itemDate[0], (parseInt(itemDate[1], 10) - 1), itemDate[2]);
      if (allDay) {
        endTime.setDate(endTime.getDate() + 1);
        endTime.setHours(0);
        endTime.setMinutes(0);
      } else {
        var endhour = calendaritem.endhour;
        if (Number.isInteger(endhour)) {
          endTime.setHours(endhour);
        } else {
          endTime.setHours(23);
        }
        var endminute = calendaritem.endminute;
        if (Number.isInteger(endminute)) {
          endTime.setMinutes(endminute);
        } else {
          endTime.setMinutes(59);
        }
      }
      var endTimeString = stamp.toISOString(endTime, {zulu: false});
      endTimeString = endTimeString.substring(0, 19);
      return endTimeString;
    }
  });
});
