/*
 * モデル項目詳細ダイアログ
 */
define([
  "dojo/_base/declare",
  "dojo/_base/array",
  "dojo/dom",
  "dojo/dom-construct",
  "dojo/dom-style",
  "dojo/query",
  "dojo/store/Memory",
  "dojo/topic",
  "dojox/widget/Standby",
  "dijit/registry",
  "dijit/form/Button",
  "dijit/form/CheckBox",
  "dijit/layout/ContentPane",
  "dijit/layout/TabContainer",
  "wb/ModelitemDetailRepository",
  "wb/ModelitemDetailSettingsByType",
  "wb/ModelitemDetailWidget",
  "wb/Dialog"
], function(
  declare,
  array,
  dom,
  domConstruct,
  domStyle,
  query,
  Memory,
  topic,
  Standby,
  registry,
  Button,
  CheckBox,
  ContentPane,
  TabContainer,
  ModelitemDetailRepository,
  ModelitemDetailSettingsByType,
  ModelitemDetailWidget,
  Dialog
) {
  return declare([Dialog, ModelitemDetailRepository, ModelitemDetailSettingsByType], {
    constructor: function(initValue) {
      this.jfcRes = initValue.jfcRes;
      this.modelitemDetailWidgetOptions = this.getModelitemDetailWidgetOptions(initValue.jfcRes);
      this.selectedSettingLabel = "";
      this.loadedTabContents = [];
      this.userDefinedItemCheckBoxes = [];
      this.errors = {};
      this.widgetsCreated = false;
      this.settingsByModelitemType = this.getSettingsByModelitemType();
      this.settingsByDisplayitemType = this.getSettingsByDisplayitemType();
      this.userDefinedItemWidgets = [];
      this.OUTLINE_STYLE = "dashed #385572";
    },

    createWidget: function(key) {
      return new ModelitemDetailWidget({
        key: key,
        container: this,
      });
    },

    initDialog: function() {
      this.inherited(arguments);
      // 一括設定ダイアログ、上下のモデル項目詳細情報を表示するボタンを用意する
      this.setAdditionalButtons();
      var that = this;
      // リポジトリ変更の通知を受ける
      this.widgetSendData = false;
      topic.subscribe("widget", function(text) {
        if (text === "sendDataStart") {
          that.widgetSendData = true;
        } else if (text === "sendDataEnd") {
          that.widgetSendData = false;
        }
      });
    },

    onDialogContentLoaded: function() {
      this.inherited(arguments);
      this.hideSettingsByLocale();
      this.initModelitemImpactAnalysis();
      this.setUserDefinedItemOnChangeEvent();
    },

    hideSettingsByLocale: function() {
      if (wagbyDesigner.locale === "ja") {
        return;
      }
      // 日本語環境以外では以下の設定値を非表示にする
      var targetId = [
        "label_model_modelitem_chartype_hiragana_2", // 入力チェック ひらがな
        "label_model_modelitem_chartype_katakana_2", // 入力チェック カタカナ
        "label_model_modelitem_chartype_hankaku_katakana_2", // 入力チェック カタカナ(半角)
        "label_model_modelitem_chartype_zenkaku_katakana_2", // 入力チェック カタカナ(全角)
        "label_presentation_displayitem_ime" // 入力制御 IME
      ];
      for (var i=0,l=targetId.length; i<l; i++) {
        var setting = dom.byId(targetId[i]);
        if (!setting) {
          continue;
        }
        domStyle.set(setting.parentNode, "display", "none");
      }
      // 入力制御タブの「入力支援」設定を非表示にする
      domStyle.set("MODELITEMDETAIL_INPUT_INPUTSUPPORT", "display", "none");
    },

    initWidgets: function() {
      // データベースの詳細のリポジトリキー
      this.databaseSettingWidgetKeys = [
        "model/modelitem/@createindex",           // インデックスを作成する
        "model/modelitem/@rdbdatatype",           // テーブル定義の型
        "model/modelitem/@rdbcolumnname",         // 物理カラム名
        "model/modelitem/unique",                 // データベースで一意制約をチェックする
        "model/modelitem/relation/@refstoretype", // 参照モデルの格納先
        "model/datalock/@name",                   // モデルの関連性 ロック名
        "action/transactionForInsert",            // トランザクション制御 新規登録
        "action/transactionForInsert_2",          // トランザクション制御 新規登録
        "action/transactionForUpdate",            // トランザクション制御 更新
        "action/transactionForUpdate_2",          // トランザクション制御 更新
        "action/transactionForDelete",            // トランザクション制御 削除
        "action/transactionForDelete_2"           // トランザクション制御 削除
      ];

      // 初期値のリポジトリキー
      this.defaultValueWidgetKeys = [
        'model/modelitem/defaultValue[@action=init]',
        'model/modelitem/defaultValue[@action=insert]',
        'model/modelitem/defaultValue[@action=update]',
        'model/modelitem/defaultValue[@action=show]',
        'model/modelitem/defaultValue[@action=select]',
        'model/modelitem/defaultValue[@action=selectForEnd]',
        'model/modelitem/defaultValue[@action=search]',
        'model/modelitem/defaultValue[@action=searchForEnd]'
      ];
      this.inherited(arguments);
    },

    tabContentLoaded: function(title) {
      // 全タブのロード完了後にリソース設定の関数が実行されるようにする
      this.loadedTabContents.push(title);
      if (this.loadedTabContents.length === this.tabCount) {
        this.onDialogContentLoaded();
      }
    },

    createStandby: function() {
      this.standby = new Standby({
        'duration': 0,
        'target': "modelitemDetailDialogContent",
        'zIndex': 1000,
      });
      document.body.appendChild(this.standby.domNode);
      domStyle.set(this.standby.domNode, "width", "100%;");
      this.standby.startup();
    },

    // ダイアログの内容部はタブを表示する
    setContent: function() {
      var that = this;
      this.tabs = [];
      // タブコンテナ
      this.tabContainer = new TabContainer({
        style: "width: 100%; height: 100%;"
      }, this.contentId);

      // 詳細設定タブ
      this.detailTab = new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_ModelitemDetailDialog_Detail, // "詳細",
        href: "wb/modelitemDetailDialog/detail.html",
        preload: true,
        preventCache: true,
      });
      this.tabs.push(this.detailTab);

      // 入力チェック設定
      this.tabs.push(new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_ModelitemDetailDialog_Check, // "入力チェック",
        href: "wb/modelitemDetailDialog/check.html",
        preload: true,
        preventCache: true,
      }));
      // 入力制御設定
      this.tabs.push(new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_ModelitemDetailDialog_Input, // "入力制御",
        href: "wb/modelitemDetailDialog/input.html",
        preload: true,
        preventCache: true,
      }));
      // 出力制御(詳細画面、一覧画面、その他)設定
      this.tabs.push(new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_ModelitemDetailDialog_Output, // "出力制御",
        href: "wb/modelitemDetailDialog/output.html",
        nested: true,
        preload: true,
        preventCache: true,
      }));
      // 検索制御設定
      this.tabs.push(new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_ModelitemDetailDialog_Search, // "検索制御",
        href: "wb/modelitemDetailDialog/search.html",
        preload: true,
        preventCache: true,
      }));
      // 初期値設定
      this.tabs.push(new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_ModelitemDetailDialog_Default, // "初期値",
        href: "wb/modelitemDetailDialog/default.html",
        preload: true,
        preventCache: true,
      }));
      // 暗黙条件設定
      this.tabs.push(new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_ModelitemDetailDialog_Implicit, // "暗黙条件",
        href: "wb/modelitemDetailDialog/implicit.html",
        preload: true,
        preventCache: true,
      }));
      // 項目ヘルプ設定
      this.tabs.push(new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_ModelitemDetailDialog_Help, // "説明",
        href: "wb/modelitemDetailDialog/help.html",
        preload: true,
        preventCache: true,
      }));
      // 項目ルール設定
      this.tabs.push(new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_UserDefinedItemSettingListPane, // 項目ルール
        href: "wb/modelitemDetailDialog/userDefinedItem.html",
        preload: true,
        preventCache: true,
      }));
      // その他設定
      this.tabs.push(new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_ModelitemDetailDialog_Other, // "その他",
        href: "wb/modelitemDetailDialog/other.html",
        preload: true,
        preventCache: true,
      }));
      // 影響分析
      this.modelitemImpactAnalysisTab = new ContentPane({
        id: "wbModelitemDetailImpactAnalysisTab",
        title: wagbyDesigner.jfcRes.LABEL_TAB_ModelitemDetailDialog_ImpactAnalysis, // "影響分析",
        href: "wb/modelitemDetailDialog/impactAnalysis.html",
        preload: true,
        preventCache: true,
      });
      this.tabs.push(this.modelitemImpactAnalysisTab);
      this.tabContainer.startup();
      for (var i=0,l=this.tabs.length; i<l; i++) {
        var tab = this.tabs[i];
        tab.on("Load", function() {
          that.tabContentLoaded(this.title);
        });
        this.tabContainer.addChild(tab);
      }
      this.tabCount = this.tabContainer.getChildren().length;
      this.tabIndexByCategory = {
        'detail': 1,
        'check': 2,
        'input': 3,
        'output': 4,
        'search': 5,
        'default': 6,
        'implicit': 7,
        'help': 8,
        'other': 9
      };
    },

    setScreenResource: function() {
      this.inherited(arguments);
      // タブのタイトルをセットする
      // 出力制御タブ
      modelitemDetailDialogOutputShow.set("title", wagbyDesigner.jfcRes.LABEL_TAB_ModelitemDetailDialogOutputShow);
      modelitemDetailDialogOutputShowlist.set("title", wagbyDesigner.jfcRes.LABEL_TAB_ModelitemDetailDialogOutputShowlist);
      modelitemDetailDialogOutputOther.set("title", wagbyDesigner.jfcRes.LABEL_TAB_ModelitemDetailDialogOutputOther);
      // 説明タブ
      modelitemDetailDialogHelpInsert.set("title", wagbyDesigner.jfcRes.LABEL_TAB_ModelitemDetailDialogHelpInsert);
      modelitemDetailDialogHelpUpdate.set("title", wagbyDesigner.jfcRes.LABEL_TAB_ModelitemDetailDialogHelpUpdate);
      modelitemDetailDialogHelpSearch.set("title", wagbyDesigner.jfcRes.LABEL_TAB_ModelitemDetailDialogHelpSearch);
      modelitemDetailDialogHelpShow.set("title", wagbyDesigner.jfcRes.LABEL_TAB_ModelitemDetailDialogHelpShow);
      // 影響分析タブ
      jfcBtnShowModelitemImpactAnalysis.set("label", wagbyDesigner.jfcRes.LABEL_CATEGORY_IMPACT_ANALYSIS_BTN_SHOW_IMPACT_ANALYSIS);
    },

    setEvent: function() {
      this.inherited(arguments);
      this.RESTRICTPARAMLIST_SIZE = 10; // 「入力制御」タブの「モデル参照(絞込)」の設定数
      this.UIFUNCTION_SIZE = 5;         // 「その他」タブの「追加ボタン」の設定数

      // ボタン表示条件のイベント設定
      this.setLinkButtonEvents();
      // データベースの詳細のイベント設定
      this.setDatabaseSettingWidgetEvent();
      // トランザクション制御スクリプト設定欄の表示・非表示を切り替えるイベント設定
      this.setTransactionScriptExecuteTimingSelectEvent();
      // SQL式のイベント設定
      this.setFormulaWidgetEvent();
      // 初期値設定の入力制御
      this.setDefaultValueTypeEvent();
      // 外部キーフラグイベント設定
      this.setForeignModelFlagEvent();
      // モデル参照(絞込)のイベント設定
      this.setRestrictparamlistWidgetEvent();
      // 設定ラベル名コピーイベント設定
      this.setCopySettingLabelOnMouseUpEvent();
      // 範囲検索のイベント設定
      this.setRangeConditionWidgetEvent();
      // 別項目の値をコピーして利用のイベント設定
      this.refcopyForSearchWidgetEvent();
      // 「その他」タブの「追加ボタン」設定のボタン機能のイベント設定
      this.setUIFunctionTypeWidgetEvent();
    },

    getModel: function() {
      return this.obj.parent.parent;
    },

    getModelitem: function() {
      return this.obj.parent;
    },

    getModelitemName: function() {
      return this.obj.parent.getName();
    },

    getModelitemLineNumber: function() {
      return this.getModelitem().getNo();
    },

    showPreviousModelitemDetailRepository: function() {
      this.showModelitemDetailRepository(this.getModelitemLineNumber() - 1);
    },

    showNextModelitemDetailRepository: function() {
      this.showModelitemDetailRepository(this.getModelitemLineNumber() + 1);
    },

    showModelitemDetailRepository: function(no) {
      // テキスト入力欄からフォーカスを外さずに項目移動ボタンを押下すると変更した
      // 値のサブミットと項目の値の読み込み処理が同時に行われ、値がリポジトリへ反
      // 映されないため、上下の項目値の読み込み処理を 300ms 遅らせる
      var modelitem = this.getModel().getModelitemByNo(no);
      if (!modelitem) {
        this.disableUpDownButton(false);
        return;
      }
      this.standby.show();
      this.showModelitemDetailRetryCount = 1;
      this.showModelitemDetail(modelitem);
    },

    showModelitemDetail: function(modelitem) {
      var that = this;
      if (this.widgetSendData) {
        if (this.showModelitemDetailRetryCount > 30) {
          this.standby.hide();
          this.disableUpDownButton(false);
          return;
        }
        // リポジトリデータ送信中のため、150ms後に再実行する
        setTimeout(function() {
          that.showModelitemDetail(modelitem);
        }, 150);
        this.showModelitemDetailRetryCount = this.showModelitemDetailRetryCount + 1;
      } else {
        this.show(modelitem.detail);
      }
    },

    setAdditionalButtons: function() {
      var that = this;
      // 上下移動ボタン
      this.buttonUp = new Button({
        label: wagbyDesigner.jfcRes.LABEL_MODELITEMDETAIL_DIALOG_BUTTON_UP,
        onClick: function() {
          that.disableUpDownButton(true);
          that.showPreviousModelitemDetailRepository();
        }
      }).placeAt(this.dialog.actionBarNode, "first");
      this.buttonDown = new Button({
        label: wagbyDesigner.jfcRes.LABEL_MODELITEMDETAIL_DIALOG_BUTTON_DOWN,
        onClick: function() {
          that.disableUpDownButton(true);
          that.showNextModelitemDetailRepository();
        }
      }).placeAt(this.dialog.actionBarNode, "first");
      // 一括設定ダイアログ表示
      this.openBulkSettingDialogButton = new Button({
        label: wagbyDesigner.jfcRes.LABEL_MODELITEMDETAIL_DIALOG_BUTTON_BULKUPDATE, // 一括設定
        onClick: function() {
          // モデル項目詳細設定ダイアログを閉じる
          that.hide();
          // 一括設定ダイアログを表示する
          wagbyDesigner.showModelitemBulkUpdateDialog();
        }
      }).placeAt(this.dialog.actionBarNode, "first");
    },

    getName: function() {
      return this.obj.getName();
    },

    setLinkButtonEvents: function() {
      var keys = this.buttonConditionKeys();
      for (var i=0,l=keys.length; i<l; i++) {
        var displayKey = keys[i];
        var conditionKey = displayKey + "Condition";
        this.setLinkButtonEvent(displayKey, conditionKey);
      }
    },

    setLinkButtonEvent: function(displayKey, conditionKey) {
      var checkWidgetItem = this.widgets[displayKey];
      var conditionWidgetItem = this.widgets[conditionKey];
      conditionWidgetItem.getKey = function() {
        return displayKey;
      };

      checkWidgetItem.widget.on("Change", function(value) {
        conditionWidgetItem.widget.set("disabled", !value);
        if (!value) {
          checkWidgetItem.sendData(checkWidgetItem, "");
        } else {
          var conditionValue = conditionWidgetItem.widget.get("value");
          if (conditionValue === "") {
            checkWidgetItem.sendData(checkWidgetItem, wagbyDesigner.jfcRes.OPTION_CHECKED);
          } else {
            checkWidgetItem.sendData(checkWidgetItem, conditionValue);
          }
        }
      });
      conditionWidgetItem.widget.on("Change", function(value) {
        conditionWidgetItem.sendData(conditionWidgetItem, value);
        // 表示条件を空にした場合、「表示する」に○を送信する
        if (value === "") {
          checkWidgetItem.sendData(checkWidgetItem, wagbyDesigner.jfcRes.OPTION_CHECKED);
        } else {
        // 表示条件に条件を設定した場合、「表示する」に条件を送信する
          checkWidgetItem.sendData(checkWidgetItem, value);
        }
      });
    },

    setRepositoryItems: function(repositoryItems) {
      this.inherited(arguments);
      // ボタン表示条件のデータ設定
      this.setLinkButtonData();
    },

    setLinkButtonData: function() {
      var keys = this.buttonConditionKeys();
      for (var i=0,l=keys.length; i<l; i++) {
        var displayKey = keys[i];
        var conditionKey = displayKey + "Condition";
        var value = this.repositoryItems[displayKey].value;
        // 「表示する」項目は設定値(○、表示条件)があればチェックを付ける
        if (value && value !== "" && value !== wagbyDesigner.jfcRes.OPTION_CHECKED) {
          this.repositoryItems[displayKey].value = wagbyDesigner.jfcRes.OPTION_CHECKED;
          this.widgets[displayKey].widget.set("value", wagbyDesigner.jfcRes.OPTION_CHECKED);
        }
        // 「表示条件」項目は設定値が○以外の文字列であれば設定をセットする
        if (value && value !== wagbyDesigner.jfcRes.OPTION_CHECKED) {
          this.repositoryItems[conditionKey].value = value;
          this.widgets[conditionKey].widget.set("value", value);
        }
        // 「表示する」チェックボックス未選択の場合、条件入力欄を入力不可にする
        if (!this.widgets[displayKey].widget.get("checked")) {
          this.widgets[conditionKey].widget.set("disabled", true);
        }
      }
    },

    buttonConditionKeys: function() {
      return [
        'presentation/displaymodel/link[@action="Add"]/@showcondition',
        'presentation/displaymodel/link[@action="Insert"]/@showcondition',
        'presentation/displaymodel/link[@action="Delete"]/@showcondition',
        'presentation/displaymodel/link[@action="Move"]/@showcondition'
      ];
    },

    setDatabaseSettingWidgetEvent: function() {
      // 「値をデータベースに保存する」設定
      var databaseTransientWidget = this.widgets["model/modelitem/@transient"].widget;
      if (!databaseTransientWidget) {
        return;
      }
      var that = this;
      databaseTransientWidget.on("click", function() {
        that.changeDatabaseSetting();
      });
      // 「参照先項目のテーブル定義の型と同じ設定を適用する」設定
      var rdbdatatypeInheritWidget = this.widgets["model/modelitem/inherit/rdbdatatype"].widget;
      if (!rdbdatatypeInheritWidget) {
        return;
      }
      rdbdatatypeInheritWidget.on("click", function() {
        that.changeRdbdatatypeInheritSetting();
      });
    },

    setTransactionScriptExecuteTimingSelectEvent: function() {
      // 「実行タイミング」変更時にスクリプト設定欄の表示・非表示を切り替える
      var transactionScriptExecuteTimingSelect = this.widgets["action/script/transactionScriptExecuteTiming"].widget;
      var scriptSettingNodes;
      var that = this;
      transactionScriptExecuteTimingSelect.on("change", function(value) {
        if (!scriptSettingNodes) {
          scriptSettingNodes = query("table#serverSideTransactionScriptSetting tr", that.detailTab.domNode);
        }
        scriptSettingNodes.forEach(function(node) {
          domStyle.set(node, "display", "none");
        });
        domStyle.set(value, "display", "");
      });
    },

    setFormulaWidgetEvent: function() {
      var formulaWidget = this.widgets["model/modelitem/formula"].widget;
      if (formulaWidget) {
        var that = this;
        formulaWidget.on("change", function() {
          // ソート順の入力可・不可設定
          that.changeSortKeySetting();
        });
      }
    },

    // 「値をデータベースに保存する」設定値によって、
    // データベースの詳細設定欄の入力可・不可を設定する
    changeDatabaseSetting: function() {
      var databaseTransientWidget = this.widgets["model/modelitem/@transient"].widget;
      if (!databaseTransientWidget) {
        return;
      }
      var value = databaseTransientWidget.get("checked");
      for (var i=0,l=this.databaseSettingWidgetKeys.length; i<l; i++) {
        this.widgets[this.databaseSettingWidgetKeys[i]].widget.set("disabled", !value);
      }
      // ソート順の入力可・不可設定
      this.changeSortKeySetting();
    },

    changeSortKeySetting: function() {
      // 「値をデータベースに保存する」設定
      var transientWidget = this.widgets["model/modelitem/@transient"].widget;
      // 「SQL式」設定
      var formulaWidget = this.widgets["model/modelitem/formula"].widget;
      // 「ソート順」設定
      var sortKeyWidget = this.widgets["model/modelitem/@sortKey"].widget;
      var isTransient = transientWidget.get("checked");
      var formulaValue = formulaWidget.get("value");
      // 値をデータベースに保存しない、かつ SQL式未設定の場合、
      // ソート順をクリアして、入力不可とする
      if (!isTransient && (formulaValue === undefined || formulaValue === "")) {
        sortKeyWidget.set("value", "");
        sortKeyWidget.set("disabled", true);
      } else {
        sortKeyWidget.set("disabled", false);
      }
    },

    // 「参照先項目のテーブル定義の型と同じ設定を適用する」設定値によって、
    // 「テーブル定義の型」設定欄の入力可・不可を設定する
    changeRdbdatatypeInheritSetting: function() {
      var value = this.widgets["model/modelitem/inherit/rdbdatatype"].widget.get("value");
      if (value) {
        this.widgets["model/modelitem/@rdbdatatype"].widget.set("disabled", true);
      } else {
        this.widgets["model/modelitem/@rdbdatatype"].widget.set("disabled", false);
      }
    },

    // 初期値の種類で、値の入力欄の入力可・不可を切り替える
    setDefaultValueTypeEvent: function() {
      for (var i=0,l=this.defaultValueWidgetKeys.length; i<l; i++) {
        var defaultValueWidgetKey = this.defaultValueWidgetKeys[i];
        var typeWidget = this.widgets[defaultValueWidgetKey];
        if (!typeWidget) {
          continue;
        }
        typeWidget.widget.on("Change",
          this.getDefaultValueTypeWidgetOnChangeFunc(defaultValueWidgetKey));
      }
    },

    setCalendarinputWidgetEvent: function() {
      // 「日付入力方式」設定
      var calendarinputWidget = this.widgets["model/modelitem/@calendarinput"].widget;
      if (!calendarinputWidget) {
        return;
      }
      this.changeDateAutoformatDisabled(calendarinputWidget.get("value"));
      var that = this;
      calendarinputWidget.on("change", function(value) {
        that.changeDateAutoformatDisabled(value);
      });
    },

    // 「日付入力方式」の設定値によって、日付オートフォーマットの入力可・不可を設定する
    changeDateAutoformatDisabled: function(value) {
      var dateAutoformatWidget = this.widgets["model/modelitem/@dateautoformat"].widget;
      if (!dateAutoformatWidget) {
        return;
      }
      var disabled = true;
      if (value === " " || value === "") {
        disabled = false;
      }
      dateAutoformatWidget.set("disabled", disabled);
      // 直接入力以外の場合、日付オートフォーマット設定のチェックボックスを外す
      if (disabled) {
        dateAutoformatWidget.set("checked", false);
      }
    },

    getDefaultValueTypeWidgetOnChangeFunc: function(key) {
      var valueWidget = this.widgets[key + "_2"];
      return function(value) {
        var disabled;
        if (value === " " || value === "random" ||
            value === "calc" || value === "javacode") {
          disabled = false;
        } else {
          disabled = true;
        }
        if (valueWidget) {
          valueWidget.widget.set("disabled", disabled);
        }
      };
    },

    setForeignModelFlagEvent: function() {
      var outsideKeyWidgetItem = this.widgets["model/modelitem/@outsideKey"];
      var that = this;
      outsideKeyWidgetItem.widget.on("change", function(value) {
        var modelitem = that.getModelitem();
        modelitem.detail.setForeignModel(outsideKeyWidgetItem.key, value);
        // 外部キーモデルの項目選択肢を設定する
        that.initOutsideKeyRefnameWidget(value);
        // 外部キーの場合、項目一覧の行番号列にアイコンを設定する
        if (value !== "") {
          wagbyDesigner.modelPane.modelitemPaneContent.setForeignKeyIcon(modelitem);
        } else {
          wagbyDesigner.modelPane.modelitemPaneContent.removeForeignKeyIcon(modelitem);
        }
        // 関連図のモデルデータ再生成フラグをセットする
        wagbyDesigner.setRelationModelDataNeedsRefresh(true);
      });
    },

    // モデル、モデル項目の選択肢のデータとリポジトリの設定値をセットする
    initFilteringSelectWidget: function(key) {
      var repositoryKey = key;
      var widgetItem = this.widgets[repositoryKey];
      var value = this.repositoryItems[repositoryKey].value;
      var modelId = this.getModel().getId();
      if (repositoryKey === "model/modelitem/copyto/@jfcstartworkstate") {
        // 開始フロー状態の一覧表示項目名
        modelId = "jfcstartworkstate";
      } else if (repositoryKey === "model/modelitem/copyto/@jfcsuspendworkstate") {
        // 保留フロー状態の一覧表示項目名
        modelId = "jfcsuspendworkstate";
      } else if (repositoryKey.lastIndexOf("model/modelitem/relation/restrictparamlist/restrictparam/@target") === 0) {
        repositoryKey = "model/modelitem/relation/restrictparamlist/restrictparam/@target";
        // モデル参照(絞込) - 参照先モデルの項目名
        modelId = this.getModelitem().getIdref();
      }
      var that = this;
      wagbyDesigner.project.getModelitemSelectList(modelId, repositoryKey, function(list) {
        if (widgetItem.widgetOptions.type !== "filteringSelect") {
          widgetItem.setValue(value);
          return;
        }
        var store = widgetItem.widget.get("store");
        if (!store) {
          store = new Memory();
          widgetItem.widget.set("store", store);
        }
        store.setData(list);
        // 以下の設定では FilteringSelect の選択肢から自項目IDを外す
        // 「別項目の値をコピーする」、「振り仮名入力元」、「別項目の値をコピーして利用する」
        if (repositoryKey === "modelitem/directcopy" || repositoryKey === "model/modelitem/inputsupport/furigana/@srcitem" ||
            repositoryKey === "modelitem/refcopyForSearch1" || repositoryKey === "modelitem/refcopyForSearch2") {
          store.remove(that.getModelitemName());
        }
        widgetItem.setValue(value);
      });
    },

    changeNotnullWidgetDisabled: function() {
      var value = this.widgets["model/modelitem/@primaryKey"].widget.get("checked");
      // 必須チェック設定項目
      var notnullWidget = this.widgets["model/modelitem/notnull"].widget;
      if (value) {
        // 「主キーとして利用する」設定がチェックされた場合、入力チェックは変更不可とする
        notnullWidget.set("disabled", true);
      } else {
        notnullWidget.set("disabled", false);
      }
    },

    setMultipleEvent: function() {
      var multipleWidget = this.widgets["model/modelitem/@multiple"].widget;
      if (!multipleWidget) {
        return;
      }
      var that = this;
      multipleWidget.on("click", function() {
        that.initRepeatItemContainerWidget();
      });
    },

    initRepeatItemContainerWidget: function() {
      // 「入力制御」->「繰り返し」設定項目の表示・非表示を設定する
      var repeatItemSettingDisplayProperty = this.getRepeatItemSettingDisplayProperty();
      for (var key in repeatItemSettingDisplayProperty) {
        domStyle.set(key, "display", repeatItemSettingDisplayProperty[key]);
      }
    },

    setRestrictparamlistWidgetEvent: function() {
      var that = this;
      var keyPrefix = "model/modelitem/relation/restrictparamlist/restrictparam/@source";
      for (var i=1,l=this.RESTRICTPARAMLIST_SIZE; i<=l; i++) {
        // 「入力制御」->「自モデル項目名」変更時に項目選択リストと
        // 固定値入力テキストボックスを切り替える
        this.setRestrictparamSourceWidgetEvent(i);
      }
    },

    setRestrictparamSourceWidgetEvent: function(i) {
      var that = this;
      var keyPrefix = "model/modelitem/relation/restrictparamlist/restrictparam/@";
      this.widgets[keyPrefix + "source" + i].widget.on("change", function(value) {
        // 自モデル項目名選択、固定値入力欄を切り替える
        that.changeRestrictparamWidget(keyPrefix, i, value);
      });
    },

    changeRestrictparamWidget: function(keyPrefix, i, value) {
      var source2WidgetKey = keyPrefix + "source" + i + "_2";
      var targetWidgetKey = keyPrefix + "target" + i;
      source2WidgetItem = this.widgets[source2WidgetKey];
      source2WidgetItem.widget.destroy();

      targetWidgetItem = this.widgets[targetWidgetKey];
      if (value === "itemname") { // "項目名"
        source2WidgetItem.widgetOptions.type = "filteringSelect";
      } else {
        source2WidgetItem.widgetOptions.type = "textbox";
      }
      source2WidgetItem.setWidget();
      source2WidgetItem.show();
      source2WidgetItem.widget.set("readOnly", this.getModel().isLocked());

      // 自モデル項目名、参照先項目名の選択肢をセットする
      this.initFilteringSelectWidget(source2WidgetKey);
      this.initFilteringSelectWidget(targetWidgetKey);
    },

    setCopySettingLabelOnMouseUpEvent: function() {
      var that = this;
      // マウスボタンを押下後に選択されているテキストがある場合、保存する
      this.dialog.on("MouseUp", function() {
        if (document.getSelection() && document.getSelection().anchorNode &&
             !document.getSelection().anchorNode.className) {
          if (document.getSelection() && document.getSelection().toString()) {
            that.selectedSettingLabel = document.getSelection().toString();
          } else {
            that.selectedSettingLabel = "";
          }
        }
      });
    },

    setRangeConditionWidgetEvent: function() {
      var rangeConditionWidget = this.widgets["model/modelitem/@rangeCondition"].widget;
      if (!rangeConditionWidget) {
        return;
      }
      var that = this;
      rangeConditionWidget.on("change", function(value) {
        that.showRefcopyForSearchWidget(value);
      });
    },

    showRefcopyForSearchWidget: function(value) {
      var display = "";
      if (!value) {
        display = "none";
      }
      domStyle.set("refcopyForSearch2", "display", display);
    },

    refcopyForSearchWidgetEvent: function() {
      // 範囲検索を行なう場合、「別項目の値をコピーして利用する」設定欄が二つになるため、
      // カンマで連結した値が本来のリポジトリ値にセットされるようにする
      var refcopyForSearchWidget = this.widgets["modelitem/refcopyForSearch"].widget;
      var refcopyForSearchWidget1 = this.widgets["modelitem/refcopyForSearch1"].widget;
      var refcopyForSearchWidget2 = this.widgets["modelitem/refcopyForSearch2"].widget;
      var refcopyForSearchWidgetEventFunc = function() {
        var refcopyForSearch1Value = refcopyForSearchWidget1.get("value");
        var refcopyForSearch2Value = refcopyForSearchWidget2.get("value");
        var value = "";
        if (refcopyForSearch1Value) {
          value = refcopyForSearch1Value;
        }
        if (refcopyForSearch2Value) {
          if (value) {
            value += ",";
          }
          value += refcopyForSearch2Value;
        }
        refcopyForSearchWidget.set("value", value);
      };
      refcopyForSearchWidget1.on("change", refcopyForSearchWidgetEventFunc);
      refcopyForSearchWidget2.on("change", refcopyForSearchWidgetEventFunc);
    },

    setUIFunctionTypeWidgetEvent: function() {
      var repositoryKeyPrefix = "action/uifunction";
      for (var i=1,l=this.UIFUNCTION_SIZE; i<=l; i++) {
        var repositoryKey = repositoryKeyPrefix;
        if (i > 1) {
          repositoryKey = repositoryKey + i;
        }
        this.setUIFunctionTypeWidgetOnChangeEvent(repositoryKey);
      }
    },

    setUIFunctionTypeWidgetOnChangeEvent: function(repositoryKey) {
      var typeWidgetItem = this.widgets[repositoryKey + "/@type"];
      var nameWidgetItem = this.widgets[repositoryKey + "/@name"];
      if (!typeWidgetItem || !nameWidgetItem) {
        return;
      }
      typeWidgetItem.widget.on("change", function(value) {
        // コピー、クリア選択時は、ボタン名として設定する
        var label = "";
        if (value !== "nop") {
          label = this.getOptions(value).label;
        }
        nameWidgetItem.widget.set("value", label);
      });
    },

    getTitle: function() {
      var label = this.obj.getLabel();
      var name = this.obj.getName();
      var titleLength = label.length + name.length;
      if (titleLength > 50) {
        label = wagbyDesigner.getEllipsisString(label, 25);
        name = wagbyDesigner.getEllipsisString(name, 25);
      }
      return this.title + " " +
        "[" + label + "(" + name + ")]";
    },

    hideTabExceptImpactAnalysis: function() {
      var display =
        wagbyDesigner.modelPane.modelitemPaneContent.isCollectionModel() ? "none" : "";

      for (var i=0,l=this.tabs.length; i<l; i++) {
        var tab = this.tabs[i];
        if (domStyle.set(tab.domNode, "display") === display) {
          break;
        }
        if (tab.id === "wbModelitemDetailImpactAnalysisTab") {
          if (display === "none") {
            this.tabContainer.selectChild("wbModelitemDetailImpactAnalysisTab");
          }
          continue;
        }
        domStyle.set(tab.controlButton.domNode, "display", display);
        domStyle.set(tab.domNode, "display", display);
      }
    },

    show: function(obj) {
      if (!obj) {
        return;
      }
      this.hideTabExceptImpactAnalysis();
      this.obj = obj;
      this.setRepositoryItems(this.obj.repositoryItems);
      // ダイアログのタイトルに項目名を表示する
      this.setDialogTitle();
      // 設定項目の初期化
      this.initializeWidget();
      // モデル項目の型等に応じて設定欄の表示・非表示を切り替える
      this.displaySettingsByType();
      // モデルロック時に入力不可に変更する
      this.setDialogLocked();
      // ユーザ定義項目がセットされている入力欄に枠線を表示する
      this.getUserDefinedItemSettings();
      // モデル項目のエラーを取得する
      wagbyDesigner.modelPane.getErrorModelitems();
      this.dialog.show();
      if (!this.widgetsCreated) {
        this.showWidgets();
        this.createStandby();
        this.widgetsCreated = true;
      }
      this.clearImpactAnalysisModelitemTableBody();
      this.standby.hide();
    },

    getModelitemTypeSimpleRecursive: function() {
      var typeSimple = this.getModelitemTypeSimple();
      if (typeSimple === "referModel") {
        var type = this.getModelitemType();
        if (type === "reference_lookup") {
          return type;
        }
      }
      return typeSimple;
    },

    getModelitemTypeRecursive: function() {
      var type = this.getModelitemType();
      if (type === "reference_lookup") {
        return type;
      }
      return type;
    },

    displaySettingsByType: function() {
      this.displaySettingsByModelitemType();
      this.displaySettingsByDisplayitemType();
      this.displaySettingsByModelFunction();
      this.colorSettings();
    },

    displaySettingsByModelitemType: function() {
      var simpleType = this.getModelitemTypeSimpleRecursive();
      var type = this.getModelitemTypeRecursive();
      for (var id in this.settingsByModelitemType) {
        this.displaySetting(id, this.isDisplaySettingsByModelitemType(id, simpleType, type));
      }
    },

    isDisplaySettingsByModelitemType: function(id, simpleType, type) {
      var settingsByType = this.settingsByModelitemType[id];
      if (settingsByType[type] === false) {
        return false;
      }
      if (settingsByType[simpleType] || settingsByType[type] ||
          (id !== "MODELITEMDETAIL_INPUT_TYPE_MODEL_RELATION_RESTRICT" &&
           id !== "MODELITEMDETAIL_SEARCH_TYPE_MODEL_RELATION_RESTRICT" &&
           type === "reference_lookup")) {
        return true;
      }
      return false;
    },

    displaySettingsByDisplayitemType: function() {
      var type = this.getModelitemTypeRecursive();
      var displayitemType = this.getDisplayitemType();
      for (var id in this.settingsByDisplayitemType) {
        var settingsByType = this.settingsByDisplayitemType[id];
        this.displaySetting(id, (settingsByType[displayitemType] || type === "reference_lookup"));
      }
    },

    displaySettingsByModelFunction: function() {
      var isWorkflowModel = wagbyDesigner.activeModel.isWorkflowModel();
      // 詳細 - 主キーの詳細
      this.displaySetting("MODELITEMDETAIL_DETAIL_PRIMARYKEY_DETAIL", this.isPrimaryKey());
      // 詳細 - ワークフロー
      this.displaySetting("MODELITEMDETAIL_DETAIL_WORKFLOW", isWorkflowModel);
      // 検索制御 - ワークフロー
      this.displaySetting("MODELITEMDETAIL_SEARCH_WORKFLOW", isWorkflowModel);
      // 入力制御 - 繰り返し
      this.displaySetting("MODELITEMDETAIL_INPUT_MULTIPLE", this.isDisplayMultipleSettings());
      // 入力制御 - アップロード更新
      this.displaySetting("MODELITEMDETAIL_INPUT_CSVUPLOADUPDATE", wagbyDesigner.activeModel.isUploadupdateOutput());
      // 出力制御 - その他 - CSV
      this.displaySetting("MODELITEMDETAIL_OUTPUT_OTHER_CSV", wagbyDesigner.activeModel.isCsvOutput());
      // 出力制御 - その他 - カレンダビュー
      this.displaySetting("MODELITEMDETAIL_OUTPUT_OTHER_CALENDARVIEW", wagbyDesigner.activeModel.isCalendarOutput());
    },

    isDisplayMultipleSettings: function() {
      if (this.isMultipleItem()) {
        return true;
      }
      var type = this.getModelitemType();
      if (type === "container" || type === "containerId") {
        return true;
      }
      var modelitem = this.getModelitem();
      if (modelitem.isContainerItem()) {
        return true;
      }
      return false;
    },

    displaySetting: function(id, isDisplay) {
      var node = dom.byId(id);
      if (!node) {
        console.warn("displaySetting id NOT exists: ", id);
        return;
      }
      var display = "";
      if (!isDisplay) {
        display = "none";
      }
      domStyle.set(node, "display", display);
    },

    colorSettings: function() {
      var tabs = this.tabContainer.getChildren();
      for (var i=0,l=tabs.length; i<l; i++) {
        query("table.dialog_table > tbody > tr:not(tr[style*='display: none;'])", tabs[i].domNode).forEach(function(node, i) {
          var bgColor = "#E6E6E6";
          if (i % 2 === 0) {
            bgColor = "#F8F8F8";
          }
          domStyle.set(node, "background", bgColor);
        });

      }
    },

    setDialogLocked: function() {
      if (!this.getModel()) {
        return;
      }
      this.openBulkSettingDialogButton.set("disabled", this.getModel().isLocked());
    },

    // 設定項目の初期化
    initializeWidget: function() {
      // データベースの詳細の表示設定
      this.initDatabaseSettingWidget();
      // 入力制御の繰り返しの表示設定
      this.initRepeatItemContainerWidget();
      // 初期値の入力可・不可を設定する
      this.initDefaultValueWidget();
      // 主キーの場合、入力チェックは変更不可とする
      this.changeNotnullWidgetDisabled();
      // 日付オートフォーマットの入力可・不可を設定する
      this.setCalendarinputWidgetEvent();
      // 検索制御の別項目の値をコピーして利用する設定
      this.initRefcopyForSearchWidget();
      // モデル名選択肢、モデル項目選択肢のデータをセットする
      this.initFilteringSelectWidgets();
      // モデル参照(絞込)の設定
      this.initRestrictparamWidget();
      // トランザクション制御時のスクリプト実行タイミング
      this.initTransactionScriptExecuteTimingSelectWidget();
      // ユーザ定義項目データの表示
      this.initUserDefinedItemList();
      // 設定エラー項目の表示
      this.setWidgetErrors();
      // 上下のモデル項目詳細情報を表示するボタンの押下制御
      this.setUpDownButtonEnabled();
    },

    setWidgetErrors: function() {
      for (var modelitemName in this.errors) {
        if (modelitemName !== this.getModelitemName()) {
          // 現在表示中の項目とは別の項目のエラーの場合、エラーを解除する
          this.unsetWidgetErrors();
          continue;
        }
        var errors = this.errors[modelitemName];
        for (var i=0,l=errors.length; i<l; i++) {
          var error = errors[i];
          var widget = this.widgets[error.key];
          if (widget) {
            widget.setWidgetError({
              message: error.errorMessage,
              notsaved: error.notsaved
            });
          }
          // エラーの入力欄があるタブの色を変更する
          this.changeCategoryTabError(error.key);
        }
      }
    },

    unsetWidgetErrors: function() {
      for (var modelitemName in this.errors) {
        var errors = this.errors[modelitemName];
        for (var i=0,l=errors.length; i<l; i++) {
          var error = errors[i];
          var widget = this.widgets[error.key];
          if (widget) {
            widget.unsetWidgetError(error.key, error.modelitemName);
          }
          this.changeCategoryTabNormal(error.key);
        }
      }
    },

    setErrors: function(errors) {
      for (var modelitemName in errors) {
        if (modelitemName && modelitemName !== "") {
          this.errors[modelitemName] = errors[modelitemName];
        }
      }
    },

    clearErrors: function() {
      if (Object.keys(this.errors).length === 0) {
        return;
      }
      // 入力欄に設定されていたエラー表示を解除する
      this.unsetWidgetErrors();
      this.errors = {};
    },

    changeCategoryTabError: function(repositoryKey) {
      var widgetOptions = this.modelitemDetailWidgetOptions[repositoryKey];
      if (!widgetOptions) {
        return;
      }
      if (!widgetOptions.category) {
        return;
      }
      var errorTabIndex = this.tabIndexByCategory[widgetOptions.category];
      if (!errorTabIndex) {
        return;
      }
      wagbyDesigner.setTabColorError(this.tabContainer, errorTabIndex);
    },

    changeCategoryTabNormal: function(repositoryKey) {
      wagbyDesigner.setTabColorNormal(this.tabContainer);
    },

    initFilteringSelectWidgets: function() {
      // 外部キーモデル選択肢設定
      this.initOutsideKeyWidget();
      // リンク表示に利用する項目名の選択肢をセットする
      this.initFilteringSelectWidget("model/modelitem/urlinfo/@displayfield");
      // 開始フロー状態の一覧表示項目名
      this.initFilteringSelectWidget("model/modelitem/copyto/@jfcstartworkstate");
      // 保留フロー状態の一覧表示項目名
      this.initFilteringSelectWidget("model/modelitem/copyto/@jfcsuspendworkstate");
      // 入力制御 - 共通 - 「別項目の値をコピーする」項目の選択肢を設定する
      this.initFilteringSelectWidget("modelitem/directcopy");
      // 入力制御 - 入力支援 - 振り仮名入力元項目の選択肢を設定する
      this.initFilteringSelectWidget("model/modelitem/inputsupport/furigana/@srcitem");
      // 検索制御 - 共通 - 「別項目の値をコピーして利用する」項目の選択肢を設定する
      this.initFilteringSelectWidget("modelitem/refcopyForSearch1");
      this.initFilteringSelectWidget("modelitem/refcopyForSearch2");
      // 検索制御 - 既読/未読管理
      this.initFilteringSelectWidget("action/condition/@unreadStatusSearch");
    },

    initOutsideKeyWidget: function() {
      var reposiotryKey = "model/modelitem/@outsideKey";
      var widgetItem = this.widgets[reposiotryKey];
      var value = this.repositoryItems[reposiotryKey].value;
      var that = this;
      wagbyDesigner.project.getModelListStore(this.getModel().getId(), reposiotryKey, function(store) {
        // 外部キーの場合は、アイコン表示するため、初期値設定で onChange イベントを
        // 発生させないよう関連する「参照先項目名」の設定も併せてセットする
        widgetItem.widget.set("store", store);
        widgetItem.setValue(value);
        that.initOutsideKeyRefnameWidget(value);
      });
    },

    initOutsideKeyRefnameWidget: function(modelId) {
      var repositoryKey = "model/modelitem/@outsideKeyRefname";
      var widgetItem = this.widgets[repositoryKey];
      var value = this.repositoryItems[repositoryKey].value;
      var that = this;
      wagbyDesigner.project.getModelitemSelectList(modelId, repositoryKey, function(list) {
        widgetItem.widget.get("store").setData(list);
        widgetItem.setValue(value);
      });
    },

    initRestrictparamWidget: function() {
      var keyPrefix = "model/modelitem/relation/restrictparamlist/restrictparam/@";
      for (var i=1,l=this.RESTRICTPARAMLIST_SIZE; i<=l; i++) {
        // 自モデル項目名選択、固定値入力欄を切り替える
        this.changeRestrictparamWidget(keyPrefix, i,
          this.widgets[keyPrefix + "source" + i].widget.get("value"));
      }
    },

    initTransactionScriptExecuteTimingSelectWidget: function() {
      var transactionScriptExecuteTimingSelect = this.widgets["action/script/transactionScriptExecuteTiming"].widget;
      transactionScriptExecuteTimingSelect.set("value", transactionScriptExecuteTimingSelect.get("value"));
    },

    initRefcopyForSearchWidget: function() {
      // 範囲検索を行なう場合、「別項目の値をコピーして利用する」設定欄を二つ表示する
      this.showRefcopyForSearchWidget(this.widgets["model/modelitem/@rangeCondition"].widget.get("checked"));
      // 範囲検索を行なう場合、設定値がカンマ区切りとなっているため、
      // 二つの設定欄に設定値をセットする
      var value = this.repositoryItems["modelitem/refcopyForSearch"].value;
      if (!value) {
        return;
      }
      var values = value.split(",");
      var repositoryKeyPrefix = "modelitem/refcopyForSearch";
      for (var i=0,l=values.length; i<l; i++) {
        var repositoryKey = repositoryKeyPrefix + (i + 1);
        var refcopyForSearchWidgetItem = this.widgets[repositoryKey];
        if (!refcopyForSearchWidgetItem) {
          continue;
        }
        refcopyForSearchWidgetItem.setValue(values[i]);
        this.repositoryItems[repositoryKey].value = value;
      }
    },

    initUserDefinedItemList: function() {
      var that = this;
      wagbyDesigner.project.getUserDefinedSettingData(function(userDefinedItemSettings) {
        that.createUserDefinedItemList(userDefinedItemSettings);
      });
    },

    setUserDefinedItemValue: function() {
      var widgets = registry.findWidgets(dom.byId("userDefinedItemList"));
      widgets = array.filter(widgets, function(widget) {
        return widget.checked;
      });
      var checkedValue = "";
      if (widgets.length > 0) {
        checkedValue = widgets[0].value;
        for (var i=1,l=widgets.length; i<l; i++) {
          checkedValue = checkedValue + "," + widgets[i].value;
        }
      }
      this.widgets["model/modelitem/userDefinedItem"].widget.set("value", checkedValue);
    },

    createUserDefinedItemList: function(userDefinedItemSettings) {
      this.clearUserDefinedItemList();
      var that = this;
      for (var key in userDefinedItemSettings) {
        var userDefinedItemSetting = userDefinedItemSettings[key];
        var value = userDefinedItemSetting["model/userDefinedSetting/@id"];
        var id = "jfcUserDefinedItemSetting" + value;
        var description = userDefinedItemSetting["model/userDefinedSetting/@description"];
        var outline = userDefinedItemSetting["model/userDefinedSetting/@outline"];
        var label = "<b>" + description + "</b>";
        var userDefinedItemContainer = domConstruct.create("input", { id: id }, "userDefinedItemList");
        var checkbox = new CheckBox({
          name: id,
          value: value,
          onClick: function() {
            that.setUserDefinedItemValue();
          }
        }, id);
        this.userDefinedItemCheckBoxes.push(checkbox);
        domConstruct.create("label", { 'for': id, innerHTML: label }, "userDefinedItemList");
        domConstruct.create("br", {}, "userDefinedItemList");
        domConstruct.create("div", { style: 'padding: 3px 0 0 25px; margin-bottom: 15px;', innerHTML: outline }, "userDefinedItemList");
        checkbox.startup();
      }
      this.setUserDefinedItemChecked();
    },

    setUserDefinedItemChecked: function() {
      var checkedValues = this.widgets["model/modelitem/userDefinedItem"].widget.get("value");
      if (checkedValues) {
        checkedValues = checkedValues.split(",");
      }
      for (var i=0,l=checkedValues.length; i<l; i++) {
        for (var j=0,m=this.userDefinedItemCheckBoxes.length; j<m; j++) {
          if (checkedValues[i] === this.userDefinedItemCheckBoxes[j].value) {
            this.userDefinedItemCheckBoxes[j].set("checked", true);
            break;
          }
        }
      }
      for (var i=0,l=this.userDefinedItemCheckBoxes.length; i<l; i++) {
        this.userDefinedItemCheckBoxes[i].set("readOnly", wagbyDesigner.activeModel.isLocked());
      }
    },

    clearUserDefinedItemList: function() {
      for (var i=0,l=this.userDefinedItemCheckBoxes.length; i<l; i++) {
        var checkbox = this.userDefinedItemCheckBoxes[i];
        if (checkbox) {
          checkbox.destroy();
        }
      }
      this.userDefinedItemCheckBoxes = [];
      domConstruct.empty("userDefinedItemList");
    },

    getUserDefinedItemSettings: function() {
      this.clearMarkUserDefinedItemSettings();
      var userDefinedItem = this.widgets["model/modelitem/userDefinedItem"].widget.get("value");
      if (!userDefinedItem || userDefinedItem === "") {
        return;
      }
      var userDefiendItemValues = userDefinedItem.split(",");
      for (var i=0,l=userDefiendItemValues.length; i<l; i++) {
        this.getUserDefinedItemSetting(userDefiendItemValues[i]);
      }
    },

    getUserDefinedItemSetting: function(userDefiendItemValue) {
      var that = this;
      wagbyDesigner.sendRequest({
        url: "/model/userDefinedItemSetting/" + userDefiendItemValue,
        options: {
          handleAs: "json"
        }
      }, function(userDefinedItemSettings) {
        that.setMarkUserDefinedItemSettings(userDefinedItemSettings);
      }, function(error) {
        return wagbyDesigner.jfcRes.MSG_ERROR_USERDEFINEDITEM_DIALOG_001; // "ユーザ定義情報の取得中にエラーが発生しました。"
      });
    },

    setMarkUserDefinedItemSettings: function(userDefinedItemSettings) {
      if (!userDefinedItemSettings.userDefinedItems) {
        return;
      }
      for (var key in userDefinedItemSettings.userDefinedItems) {
        var userDefinedItem = userDefinedItemSettings.userDefinedItems[key];
        var repositoryKeyValue = userDefinedItem["model/userDefinedItem/@repositoryKey"];
        var repositoryKey = repositoryKeyValue.split(":")[0];
        var widgetItem = this.widgets[repositoryKey];
        if (widgetItem) {
          this.setOutlineToWidget(widgetItem.widget, true);
          this.userDefinedItemWidgets.push(widgetItem);
        }
      }
    },

    clearMarkUserDefinedItemSettings: function() {
      // ユーザ定義項目がセットされている入力欄の枠線を消去する
      for (var i=0,l=this.userDefinedItemWidgets.length; i<l; i++) {
        this.setOutlineToWidget(this.userDefinedItemWidgets[i].widget, false);
      }
    },

    setOutlineToWidget: function(widget, isUserDefinedItem) {
      domStyle.set(widget.domNode, "outline", isUserDefinedItem ? this.OUTLINE_STYLE : "");
      widget.set("readOnly", isUserDefinedItem);
    },

    setUpDownButtonEnabled: function() {
      var modelitemLineNumber = this.getModelitemLineNumber();
      if (modelitemLineNumber === 1) {
        // 上ボタンを押下不可にする
        this.buttonUp.set("disabled", true);
      } else {
        this.buttonUp.set("disabled", false);
      }
      if (modelitemLineNumber === this.getModel().getModelitemCount()) {
        // 下ボタンを押下不可にする
        this.buttonDown.set("disabled", true);
      } else {
        this.buttonDown.set("disabled", false);
      }
    },

    disableUpDownButton: function(disabled) {
      this.buttonUp.set("disabled", disabled);
      this.buttonDown.set("disabled", disabled);
    },

    getRepeatItemSettingDisplayProperty: function() {
      var repeatItemSettingDisplayProperty = {
        "repeatItemContainerSettings": "none",      // 初期表示数、繰り返し項目の自動増減
        "restrictContainerSettings": "none",        // コンテナの参照連動時の表示条件
        "repeatItemAddButtonSettings": "none",      // 追加ボタン
        "repeatItemInsertButtonSettings": "none",   // 挿入ボタン
        "repeatItemDeleteButtonSettings": "none",   // 削除ボタン
        "repeatItemMoveButtonSettings": "none",     // 移動ボタン
        "containerScrollbarSettings": "none",       // コンテナ内に横スクロールバーを表示する
        "copyRepeatItemAtAdditionSettings": "",     // 追加時に値をコピーする
        "sortRepeatItemInContainerSettings": "none" // コンテナ内でソートする
      };
      if (this.isMultipleItem()) {
        // 「繰り返しとして利用する」がチェックされている場合
        repeatItemSettingDisplayProperty["repeatItemAddButtonSettings"] = "";
        repeatItemSettingDisplayProperty["repeatItemContainerSettings"] = "";
      }
      var modelitem = this.getModelitem();
      var type = this.getModelitemType();
      if (type === "container") {
        repeatItemSettingDisplayProperty["containerScrollbarSettings"] = "";
        repeatItemSettingDisplayProperty["repeatItemInsertButtonSettings"] = "";
        repeatItemSettingDisplayProperty["repeatItemDeleteButtonSettings"] = "";
        repeatItemSettingDisplayProperty["repeatItemMoveButtonSettings"] = "";
        repeatItemSettingDisplayProperty["restrictContainerSettings"] = "";
        repeatItemSettingDisplayProperty["copyRepeatItemAtAdditionSettings"] = "none";
      } else if (type === "containerId") {
        repeatItemSettingDisplayProperty["copyRepeatItemAtAdditionSettings"] = "none";
      } else if (modelitem.isContainerItem()) {
        // コンテナ内の項目の場合、「コンテナ内でソートする」設定欄を表示する
        repeatItemSettingDisplayProperty["sortRepeatItemInContainerSettings"] = "";
      }
      return repeatItemSettingDisplayProperty;
    },

    initDatabaseSettingWidget: function() {
      // 参照連動の場合、「参照モデルの格納先」設定欄を表示する
      var type = this.getModelitemType();
      var display = "none";
      if (type === "reference_lookup") {
        display = "";
      }
      domStyle.set(dom.byId("ModelitemRelationRefstoretypeRow"), "display", display);

      // 「データベースに値を保存する」設定値によって、入力可・不可を設定する
      this.changeDatabaseSetting();
      // 「参照先項目のテーブル定義の型と同じ設定を適用する」設定は他のモデル参照の場合のみ入力可とする
      var rdbdatatypeInheritWidget = this.widgets["model/modelitem/inherit/rdbdatatype"].widget;
      var typeSimple = this.getModelitemTypeSimple();
      if (typeSimple === "referModel") {
        rdbdatatypeInheritWidget.set("disabled", false);
      } else {
        rdbdatatypeInheritWidget.set("disabled", true);
      }
      // 「参照先項目のテーブル定義の型と同じ設定を適用する」設定値によって、
      // 「テーブル定義の型」設定欄の入力可・不可を設定する
      this.changeRdbdatatypeInheritSetting();
    },

    initDefaultValueWidget: function() {
      for (var i=0,l=this.defaultValueWidgetKeys.length; i<l; i++) {
        var defaultValueWidgetKey = this.defaultValueWidgetKeys[i];
        var typeWidget = this.widgets[defaultValueWidgetKey];
        if (!typeWidget) {
          continue;
        }
        typeWidget.widget.onChange(typeWidget.widget.get("value"));
      }
    },

    initModelitemImpactAnalysis: function() {
      this.msgImpactAnalysisModelitem = dom.byId("jfcMsgImpactAnalysisModelitem");
      this.impactAnalysisModelitemTableBody = dom.byId("jfcImpactAnalysisModelitemTableBody");
      this.impactAnalysisModelitemTable = this.impactAnalysisModelitemTableBody.parentNode;
      // 影響分析ボタン押下時の処理
      jfcBtnShowModelitemImpactAnalysis.on("click", this.setBtnShowModelitemImpactAnalysisClickEvent());
    },

    setUserDefinedItemOnChangeEvent: function() {
      var that = this;
      this.widgets["model/modelitem/userDefinedItem"].widget.on("change", function() {
        that.getUserDefinedItemSettings();
      });
    },

    setBtnShowModelitemImpactAnalysisClickEvent: function() {
      var that = this;
      return function() {
        that.standby.show();
        that.clearImpactAnalysisModelitemTableBody();
        wagbyDesigner.sendRequest({
          'url': that.getImpactAnalysisUrl(),
          'options': {
            'handleAs': "json",
          },
        }, function(data) {
          that.displayImpactAnalysis(data);
          that.standby.hide();
        }, function(err) {
          that.standby.hide();
        });
      }
    },

    getImpactAnalysisUrl: function() {
      return "/analyzeImpact/model/" + this.getModel().getId()
        + "/modelitem/" + this.getModelitem().getName();
    },

    displayImpactAnalysis: function(data) {
      var relatedModels = data.updateModelitemInfo.modifyMapOtherModel;
      if (Object.keys(relatedModels).length === 0) {
        this.setMessage(this.jfcRes.MSG_CATEGORY_IMPACT_ANALYSIS_NO_MODELITEM_FOUND);
        return;
      }
      for (var modelId in relatedModels) {
        // モデル名表示
        var model = wagbyDesigner.project.getModelById(modelId);
        var tr = domConstruct.create("tr", {}, this.impactAnalysisModelitemTableBody);
        this.createAnalysisModelTd(modelId, tr);
        // モデル項目名表示
        this.createRelatedModelitemNames(modelId, relatedModels[modelId].modelitemMap);
        // スクリプト名表示
        this.createRelatedScriptFilenames(relatedModels[modelId].scriptMap);
      }
      this.showImpactAnalysisModelitemTable();
    },

    createAnalysisModelTd: function(modelId, tr) {
      var td = domConstruct.create("td", { 'colspan': "2" }, tr);
      wagbyDesigner.getModelDescription(modelId, function(description) {
        td.textContent = description;
      });
    },

    createAnalysisModelitemTd: function(modelId, modelitemName, tr) {
      var td = domConstruct.create("td", {}, tr);
      wagbyDesigner.getModelitemLabelWithName(modelId, modelitemName, function(label) {
        td.textContent = label;
      });
    },

    createRelatedModelitemNames: function(modelId, relatedModelitemNames) {
      for (var modelitemName in relatedModelitemNames) {
        this.createAnalysisModelitemTd(modelId, modelitemName, this.createAnalysisTableRow());
      }
    },

    createRelatedScriptFilenames: function(relatedScriptFilenames) {
      for (var scriptFilename in relatedScriptFilenames) {
        var td = this.createAnalysisTableColumn();
        td.textContent = scriptFilename + ": " + relatedScriptFilenames[scriptFilename];
      }
    },

    createAnalysisTableRow: function() {
      var tr = domConstruct.create("tr", {}, this.impactAnalysisModelitemTableBody);
      domConstruct.create("td", {}, tr);
      return tr;
    },

    createAnalysisTableColumn: function() {
      return domConstruct.create("td", {}, this.createAnalysisTableRow());
    },

    clearImpactAnalysisModelitemTableBody: function() {
      this.hideImpactAnalysisModelitemTable();
      domConstruct.empty(this.impactAnalysisModelitemTableBody);
      this.setMessage("");
    },

    showImpactAnalysisModelitemTable: function() {
      this.setImpactAnalysisModelitemTableDisplay("");
    },

    hideImpactAnalysisModelitemTable: function() {
      this.setImpactAnalysisModelitemTableDisplay("none");
    },

    setImpactAnalysisModelitemTableDisplay: function(display) {
      domStyle.set(this.impactAnalysisModelitemTable, "display", display);
    },

    setMessage: function(message) {
      this.msgImpactAnalysisModelitem.textContent = message;
    },

    hide: function() {
      this.clearMarkUserDefinedItemSettings();
      // モデル項目のエラーを取得する
      wagbyDesigner.modelPane.getErrorModelitems();
      this.inherited(arguments);
    },

    getRepositoryItem: function(key) {
      return this.inherited(arguments);
    },

    getModelitemType: function() {
      return this.getModelitem().repositoryItems["model/modelitem/@type"].value;
    },

    getModelitemTypeSimple: function() {
      return this.getModelitem().repositoryItems["model/modelitem/@typeSimple"].value;
    },

    getDisplayitemType: function() {
      return this.getModelitem().getDisplayitemType();
    },

    isPrimaryKey: function() {
      return this.getModelitem().isPrimaryKey();
    },

    isMultipleItem: function() {
      return this.getModelitem().isMultipleItem();
    },

  });
});
